@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 
 """
-Copyright (c) 2006-2018 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -20,7 +20,7 @@ def dependencies():
 
 def tamper(payload, **kwargs):
     """
-    Replaces plus ('+') character with function CONCAT()
+    Replaces plus operator ('+') with (MsSQL) function CONCAT() counterpart
 
     Tested against:
         * Microsoft SQL Server 2012
@@ -41,6 +41,9 @@ def tamper(payload, **kwargs):
     retVal = payload
 
     if payload:
+        prefix, suffix = '+' * len(re.search(r"\A(\+*)", payload).group(0)), '+' * len(re.search(r"(\+*)\Z", payload).group(0))
+        retVal = retVal.strip('+')
+
         while True:
             indexes = zeroDepthSearch(retVal, '+')
 
@@ -76,4 +79,6 @@ def tamper(payload, **kwargs):
                 else:
                     break
 
+        retVal = "%s%s%s" % (prefix, retVal, suffix)
+
     return retVal
