@@ -16,9 +16,10 @@ def detect(get_page):
     retval = False
 
     for vector in WAF_ATTACK_VECTORS:
-        _, headers, _ = get_page(get=vector)
+        page, headers, _ = get_page(get=vector)
         retval = re.search(r"YUNDUN", headers.get(HTTP_HEADER.SERVER, ""), re.I) is not None
         retval |= re.search(r"YUNDUN", headers.get("X-Cache", ""), re.I) is not None
+        retval |= "Blocked by YUNDUN Cloud WAF" in (page or "")
         if retval:
             break
 
